# -*- coding: utf-8 -*-

from sys import exit
from os import getpid, listdir, sep
from modules.functions import *
from modules.functions_bot import *
from pyrogram import Client, filters, idle
from pyrogram.types import ForceReply, BotCommand, BotCommandScopeChat
from pyrogram.enums.chat_action import ChatAction


if configGet("bot_token") != "12345678:asdfghjklzxcvbnm":
    pid = getpid()
    app = Client("auto_zoom_public_bot", api_id=configGet("api_id"), api_hash=configGet("api_hash"), bot_token=configGet("bot_token"))
else:
    logWrite("Could not start the bot. Please, configure token in config.json", logs_folder=configGet("logs"))
    exit()


@app.on_message(~ filters.scheduled & filters.command(["link", "start"], prefixes="/"))
async def start(app, msg):

    logWrite(f'Got command start/link from {msg.from_user.id}', logs_folder=configGet("logs"))

    await app.send_chat_action(chat_id=msg.chat.id, action=ChatAction.TYPING)

    user_locale = msg.from_user.language_code
    data = configGet("data")

    if f"{msg.from_user.id}.json" not in listdir(f"{data}{sep}users{sep}"):
        logWrite(f'Creating blank data file for {msg.from_user.id}', logs_folder=configGet("logs"))
        jsonSave( f"{data}{sep}users{sep}{msg.from_user.id}.json", {"api_key": None, "linked": False, "context": {"action": None, "data": None}} )

    if not userGet(msg.from_user.id, "linked"):
        await msg.reply_text(locale("link_input", "msg", locale=user_locale), reply_markup=ForceReply(placeholder=locale("link", "fry", locale=user_locale)))
        userSet(msg.chat.id, "context", "link_key")
    else:
        await msg.reply_text(locale("already_linked", "msg", locale=user_locale))

@app.on_message(~ filters.scheduled & filters.command(["unlink"], prefixes="/"))
async def unlink(app, msg):

    logWrite(f'Got command ulink from {msg.from_user.id}', logs_folder=configGet("logs"))

    await app.send_chat_action(chat_id=msg.chat.id, action=ChatAction.TYPING)

    user_locale = msg.from_user.language_code
    data = configGet("data")

    if not userGet(msg.from_user.id, "linked"):
        await msg.reply_text(locale("not_linked", "msg", locale=user_locale))
    else:
        try:
            keys_storage = jsonLoad(f"{data}{sep}keys_storage.json")
            del keys_storage[userGet(msg.from_user.id, "api_key")]
            jsonSave(f"{data}{sep}keys_storage.json", keys_storage)
        except:
            pass
        userClear(msg.from_user.id, "api_key")
        userSet(msg.chat.id, "linked", False)
        await msg.reply_text(locale("unlinked", "msg", locale=user_locale))

@app.on_message(~ filters.scheduled & filters.command(["cancel"], prefixes="/"))
async def cancel(app, msg):

    await app.send_chat_action(chat_id=msg.chat.id, action=ChatAction.TYPING)

    user_locale = msg.from_user.language_code

    if userGet(msg.from_user.id, "context") is not None:
        userClear(msg.from_user.id, "context")
        userClear(msg.from_user.id, "context_content")
        await msg.reply_text(locale("cancel", "msg", locale=user_locale))
    else:
        await msg.reply_text(locale("cancel_empty", "msg", locale=user_locale))


@app.on_message(filters.command(["kill", "die", "shutdown", "reboot"], prefixes="/"))
async def kill(app, msg):

    if msg.from_user.id == configGet("admin"):
        await msg.reply_text(f"Shutting down bot with pid `{pid}`")
        killProc(pid)


@app.on_message(~ filters.scheduled)
async def any_message_handler(app, msg):

    if userGet(msg.from_user.id, "context") == "link_key":

        user_locale = msg.from_user.language_code
        data = configGet("data")

        if msg.text in jsonLoad(configGet("api_keys"))["autozoom"]:
            await msg.reply_text(locale("key_correct", "msg", locale=user_locale))
            userSet(msg.from_user.id, "api_key", msg.text)
            userSet(msg.from_user.id, "linked", True)
            keys_storage = jsonLoad(f"{data}{sep}keys_storage.json")
            keys_storage[msg.text] = msg.from_user.id
            jsonSave(f"{data}{sep}keys_storage.json", keys_storage)
            logWrite(f"Added apikey {msg.text} for user {msg.from_user.id}", logs_folder=configGet("logs"))
        elif msg.text in jsonLoad(configGet("expired_keys")):
            logWrite(f"User {msg.from_user.id} tried to pair with expired apikey {msg.text}", logs_folder=configGet("logs"))
            await msg.reply_text(locale("key_expired", "msg", locale=user_locale))
        else:
            logWrite(f"User {msg.from_user.id} tried to pair with invalid apikey {msg.text}", logs_folder=configGet("logs"))
            await msg.reply_text(locale("key_wrong", "msg", locale=user_locale))

        userClear(msg.from_user.id, "context")
        userClear(msg.from_user.id, "context_content")


if __name__ == "__main__":

    logWrite(f'Starting with PID {str(pid)}', logs_folder=configGet("logs"))

    app.start()
    app.send_message(configGet("admin"), f"Starting bot with pid `{pid}`")

    locales = configGet("locales")
    
    for entry in listdir(locales):

        if entry.endswith(".json"):

            all_commands = locale("cmd", locale=entry.replace(".json", ""))

            # Registering user commands
            commands_list = []
            for command in all_commands["general"]:
                commands_list.append(BotCommand(command, all_commands["general"][command]))
            app.set_bot_commands(commands_list, language_code=entry.replace(".json", ""))

            # Registering admin commands
            commands_admin_list = []
            for command in all_commands["general"]:
                commands_admin_list.append(BotCommand(command, all_commands["general"][command]))
            for command in all_commands["admin"]:
                commands_admin_list.append(BotCommand(command, all_commands["admin"][command]))
            app.set_bot_commands(commands_admin_list, language_code=entry.replace(".json", ""), scope=BotCommandScopeChat(chat_id=configGet("admin")))

    idle()

    app.send_message(configGet("admin"), f"Shutting down bot with pid `{pid}`")
    logWrite(f'Shutting down with PID {pid}', logs_folder=configGet("logs"))

    killProc(pid)